@@ -52,7 +52,7 @@ void DataImpl ::delete_server(Quark const &server_in)
 
   if (_servers.count (server))
   {
-    const std::string newsrc_filename (_servers[server].newsrc_filename);
+    const std::string newsrc_filename (file::absolute_fn("",_servers[server].newsrc_filename));
     _servers.erase (server);
     save_server_properties (*_data_io, _prefs);
     std::remove (newsrc_filename.c_str());
@@ -501,7 +501,7 @@ namespace
       if (s.newsrc_filename.empty())
       { // set a default filename
         std::ostringstream o;
-        o << file::get_pan_home() << G_DIR_SEPARATOR << "newsrc-" << it->first;
+        o << "newsrc-" << it->first;
         s.newsrc_filename = o.str();
       }
     }
