@@ -19,7 +19,8 @@
 
 #include <config.h>
 #include <glib/gi18n.h>
-#include "gtk-compat.h"
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <cctype>
 #include <cmath>
 #include <algorithm>
@@ -34,13 +35,11 @@
 #include <pan/usenet-utils/mime-utils.h>
 #include <pan/data/article.h>
 #include <pan/data/data.h>
-#include <pan/icons/pan-pixbufs.h>
 #include "header-pane.h"
+#include "pan/gui/load-icon.h"
 #include "render-bytes.h"
 #include "tango-colors.h"
-#include "gtk-compat.h"
 
-#include <pan/general/null.h>
 
 using namespace pan;
 
@@ -61,15 +60,13 @@ namespace
 namespace
 {
   // default color theme's Colors
-  const PanColors& colors (PanColors::get());
-  const char* def_color_str (colors.def_bg.c_str());
-  const char* def_color_fg_str (colors.def_fg.c_str());
+  PanColors const &colors(PanColors::get());
 }
 
-const Article*
-HeaderPane :: get_article (GtkTreeModel* model, GtkTreeIter* iter)
+Article const *HeaderPane ::get_article(GtkTreeModel *model, GtkTreeIter *iter)
 {
-  const Article * a = dynamic_cast<Row*>(PAN_TREE_STORE(model)->get_row(iter))->article;
+  Article const *a =
+    dynamic_cast<Row *>(PAN_TREE_STORE(model)->get_row(iter))->article;
   g_assert (a != nullptr);
   return a;
 }
@@ -80,55 +77,55 @@ HeaderPane :: get_article (GtkTreeModel* model, GtkTreeIter* iter)
 
 namespace
 {
-  typedef std::set<const Article*> articles_t;
+typedef std::set<Article const *> articles_t;
 
-  enum
-  {
-    ICON_READ,
-    ICON_UNREAD,
+enum
+{
+  ICON_READ,
+  ICON_UNREAD,
 
-    ICON_COMPLETE,
-    ICON_COMPLETE_READ,
+  ICON_COMPLETE,
+  ICON_COMPLETE_READ,
 
-    ICON_INCOMPLETE,
-    ICON_INCOMPLETE_READ,
+  ICON_INCOMPLETE,
+  ICON_INCOMPLETE_READ,
 
-    ICON_CACHED,
-    ICON_QUEUED,
-    ICON_ERROR,
-    ICON_EMPTY,
-    ICON_FLAGGED,
-    ICON_GET_FLAGGED,
-    ICON_QTY
-   };
+  ICON_CACHED,
+  ICON_QUEUED,
+  ICON_ERROR,
+  ICON_EMPTY,
+  ICON_FLAGGED,
+  ICON_GET_FLAGGED,
+  ICON_QTY
+};
 
-  struct Icon {
-    const guint8 * pixbuf_txt;
+struct Icon
+{
+    char const *icon_file;
     GdkPixbuf * pixbuf;
   } _icons[ICON_QTY] = {
-    { icon_article_read,           nullptr },
-    { icon_article_unread,         nullptr },
+    { "icon_article_read.png",           nullptr },
+    { "icon_article_unread.png",         nullptr },
 
-    { icon_binary_complete,        nullptr },
-    { icon_binary_complete_read,   nullptr },
+    { "icon_binary_complete.png",        nullptr },
+    { "icon_binary_complete_read.png",   nullptr },
 
-    { icon_binary_incomplete,      nullptr },
-    { icon_binary_incomplete_read, nullptr },
+    { "icon_binary_incomplete.png",      nullptr },
+    { "icon_binary_incomplete_read.png", nullptr },
 
-    { icon_disk,                   nullptr },
-    { icon_bluecheck,              nullptr },
-    { icon_x,                      nullptr },
-    { icon_empty,                  nullptr },
-    { icon_red_flag,               nullptr },
-    { icon_get_flagged,            nullptr }
+    { "icon_disk.png",                   nullptr },
+    { "icon_bluecheck.png",              nullptr },
+    { "icon_x.png",                      nullptr },
+    { "icon_empty.png",                  nullptr },
+    { "icon_red_flag.png",               nullptr },
+    { "icon_get_flagged.png",            nullptr }
   };
 
-  int
-  get_article_state (const Data& data, const Article * a)
+  int get_article_state(Data const &data, Article const *a)
   {
     int retval;
-    const bool read (data.is_read (a));
-    const int part_state (a->get_part_state());
+    bool const read(data.is_read(a));
+    int const part_state(a->get_part_state());
 
     if (part_state==Article::COMPLETE && read)
       retval = ICON_COMPLETE_READ;
@@ -146,11 +143,10 @@ namespace
     return retval;
   }
 
-  int
-  get_article_action (const Article       * article,
-                      const ArticleCache  & cache,
-                      const Queue         & queue,
-                      const Quark         & message_id)
+  int get_article_action(Article const *article,
+                         ArticleCache const &cache,
+                         Queue const &queue,
+                         Quark const &message_id)
   {
     int offset (ICON_EMPTY);
 
@@ -177,7 +173,7 @@ HeaderPane :: render_action (GtkTreeViewColumn * ,
 {
   int index (0);
   gtk_tree_model_get (model, iter, COL_ACTION, &index, -1);
-  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, NULL);
+  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, nullptr);
 }
 
 void
@@ -189,7 +185,7 @@ HeaderPane :: render_state (GtkTreeViewColumn * ,
 {
   int index (0);
   gtk_tree_model_get (model, iter, COL_STATE, &index, -1);
-  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, NULL);
+  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, nullptr);
 }
 
 int
@@ -199,9 +195,9 @@ HeaderPane :: find_highest_followup_score (GtkTreeModel * model,
   int score (-9999);
   GtkTreeIter child;
   if (gtk_tree_model_iter_children (model, &child, parent)) do {
-    const Article * a (get_article (model, &child));
-    score = std::max (score, a->score);
-    score = std::max (score, find_highest_followup_score (model, &child));
+        Article const *a(get_article(model, &child));
+        score = std::max(score, a->score);
+        score = std::max(score, find_highest_followup_score(model, &child));
   } while (gtk_tree_model_iter_next (model, &child));
   return score;
 }
@@ -225,28 +221,28 @@ HeaderPane :: render_score (GtkTreeViewColumn * ,
   HeaderPane * self (static_cast<HeaderPane*>(user_data));
   GtkTreeView * view (GTK_TREE_VIEW (self->_tree_view));
   GtkTreePath * path (gtk_tree_model_get_path (model, iter));
-  const bool expanded (gtk_tree_view_row_expanded (view, path));
+  bool const expanded(gtk_tree_view_row_expanded(view, path));
   gtk_tree_path_free (path);
   if (!expanded)
     score = std::max (score, self->find_highest_followup_score (model, iter));
 
-  const Prefs& prefs (self->_prefs);
+  Prefs const &prefs(self->_prefs);
   std::string bg, fg;
   if (score >= 9999) {
-    fg = prefs.get_color_str ("score-color-watched-fg", def_color_fg_str);
+    fg = prefs.get_color_str ("score-color-watched-fg", colors.def_fg);
     bg = prefs.get_color_str ("score-color-watched-bg", TANGO_CHAMELEON_LIGHT);
   } else if (score >= 5000) {
-    fg = prefs.get_color_str ("score-color-high-fg", def_color_fg_str);
+    fg = prefs.get_color_str ("score-color-high-fg", colors.def_fg);
     bg = prefs.get_color_str ("score-color-high-bg", TANGO_BUTTER_LIGHT);
   } else if (score >= 1) {
-    fg = prefs.get_color_str ("score-color-medium-fg", def_color_fg_str);
+    fg = prefs.get_color_str ("score-color-medium-fg", colors.def_fg);
     bg = prefs.get_color_str ("score-color-medium-bg", TANGO_SKY_BLUE_LIGHT);
   } else if (score <= -9999) {
     fg = prefs.get_color_str ("score-color-ignored-fg", TANGO_ALUMINUM_4);
-    bg = prefs.get_color_str ("score-color-ignored-bg", def_color_str);
+    bg = prefs.get_color_str ("score-color-ignored-bg", colors.def_bg);
   } else if (score <= -1) {
     fg = prefs.get_color_str ("score-color-low-fg", TANGO_ALUMINUM_2);
-    bg = prefs.get_color_str ("score-color-low-bg", def_color_str);
+    bg = prefs.get_color_str ("score-color-low-bg", colors.def_bg);
   }
   else if (score == 0)
   {
@@ -255,9 +251,9 @@ HeaderPane :: render_score (GtkTreeViewColumn * ,
   }
 
   g_object_set (renderer, "text", buf,
-                          "background", (bg.empty() ? NULL : bg.c_str()),
-                          "foreground", (fg.empty() ? NULL : fg.c_str()),
-                          NULL);
+                          "background", (bg.empty() ? nullptr : bg.c_str()),
+                          "foreground", (fg.empty() ? nullptr : fg.c_str()),
+                          nullptr);
 }
 
 void
@@ -267,13 +263,13 @@ HeaderPane :: render_author (GtkTreeViewColumn * ,
                             GtkTreeIter       * iter,
                             gpointer            user_data)
 {
-  const HeaderPane * self (static_cast<HeaderPane*>(user_data));
-  const Article * a (self->get_article (model, iter));
+  HeaderPane const *self(static_cast<HeaderPane *>(user_data));
+  Article const *a(self->get_article(model, iter));
 
   g_object_set (renderer, "text", a->author.c_str() ,
                          "background", self->_bg.c_str(),
                          "foreground", self->_fg.c_str(),
-                         NULL);
+                         nullptr);
 }
 
 void
@@ -284,7 +280,7 @@ HeaderPane :: render_lines (GtkTreeViewColumn * ,
                             gpointer            user_data)
 {
 
-  const HeaderPane * self (static_cast<HeaderPane*>(user_data));
+  HeaderPane const *self(static_cast<HeaderPane *>(user_data));
 
   unsigned long lines(0ul);
   std::stringstream str;
@@ -295,7 +291,7 @@ HeaderPane :: render_lines (GtkTreeViewColumn * ,
   g_object_set (renderer, "text", str.str().c_str(),
                 "background", self->_bg.c_str(),
                 "foreground", self->_fg.c_str(),
-                NULL);
+                nullptr);
 }
 
 void
@@ -305,7 +301,7 @@ HeaderPane :: render_bytes (GtkTreeViewColumn * ,
                             GtkTreeIter       * iter,
                             gpointer            userdata)
 {
-  const HeaderPane * self (static_cast<HeaderPane*>(userdata));
+  HeaderPane const *self(static_cast<HeaderPane *>(userdata));
 
   unsigned long bytes (0);
   gtk_tree_model_get (model, iter, COL_BYTES, &bytes, -1);
@@ -313,7 +309,7 @@ HeaderPane :: render_bytes (GtkTreeViewColumn * ,
                 "text", pan::render_bytes(bytes),
                 "background", self->_bg.c_str(),
                 "foreground", self->_fg.c_str(),
-                NULL);
+                nullptr);
 }
 
 void
@@ -323,7 +319,7 @@ HeaderPane :: render_date  (GtkTreeViewColumn * ,
                             GtkTreeIter       * iter,
                             gpointer            userdata)
 {
-  const HeaderPane * self (static_cast<HeaderPane*>(userdata));
+  HeaderPane const *self(static_cast<HeaderPane *>(userdata));
 
   gchar* date (nullptr);
   gtk_tree_model_get (model, iter, COL_DATE_STR, &date, -1);
@@ -331,7 +327,7 @@ HeaderPane :: render_date  (GtkTreeViewColumn * ,
                 "text", date,
                 "background", self->_bg.c_str(),
                 "foreground", self->_fg.c_str(),
-                NULL);
+                nullptr);
   g_free (date);
 }
 
@@ -400,7 +396,7 @@ HeaderPane :: render_subject (GtkTreeViewColumn * ,
     "text", res.c_str(),
     "weight", (bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL),
     "foreground", unread ? (def_fg.empty() ? self->_fg.c_str() : def_fg.c_str()) : self->_fg.c_str(),
-    "background", unread ? (def_bg.empty() ? self->_bg.c_str() : def_bg.c_str()) : self->_bg.c_str(), NULL);
+    "background", unread ? (def_bg.empty() ? self->_bg.c_str() : def_bg.c_str()) : self->_bg.c_str(), nullptr);
 
 }
 
@@ -495,7 +491,7 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
     case COL_BYTES: {
       const unsigned long a_bytes (row_a.article->get_byte_count());
       const unsigned long b_bytes (row_b.article->get_byte_count());
-           if (a_bytes < b_bytes) ret = -1;
+      if      (a_bytes < b_bytes) ret = -1;
       else if (a_bytes > b_bytes) ret = 1;
       else                        ret = 0;
       break;
@@ -504,7 +500,7 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
     case COL_LINES: {
       const unsigned long a_lines (row_a.article->get_line_count());
       const unsigned long b_lines (row_b.article->get_line_count());
-           if (a_lines < b_lines) ret = -1;
+      if      (a_lines < b_lines) ret = -1;
       else if (a_lines > b_lines) ret = 1;
       else                        ret = 0;
       break;
@@ -517,7 +513,7 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
     default: { // COL_DATE
       const time_t a_time (row_a.article->time_posted);
       const time_t b_time (row_b.article->time_posted);
-           if (a_time < b_time) ret = -1;
+      if      (a_time < b_time) ret = -1;
       else if (a_time > b_time) ret = 1;
       else                      ret = 0;
       break;
@@ -565,11 +561,11 @@ HeaderPane :: build_model (const Quark               & group,
 
   GtkTreeSortable * sort = GTK_TREE_SORTABLE(store);
   for (int i=0; i<N_COLUMNS; ++i)
-    gtk_tree_sortable_set_sort_func (sort, i, column_compare_func, GINT_TO_POINTER(i), NULL);
+    gtk_tree_sortable_set_sort_func (sort, i, column_compare_func, GINT_TO_POINTER(i), nullptr);
   if (!group.empty()) {
     const EvolutionDateMaker date_maker;
     const bool do_thread (_prefs.get_flag ("thread-headers", true));
-    add_children_to_model (store, NULL, Quark(), atree, date_maker, do_thread);
+    add_children_to_model (store, nullptr, Quark(), atree, date_maker, do_thread);
   }
 
   return store;
@@ -602,7 +598,7 @@ HeaderPane :: rebuild ()
   }
 
   _mid_to_row.clear ();
-  _tree_store = build_model (_group, _atree, NULL);
+  _tree_store = build_model (_group, _atree, nullptr);
 
   const bool sort_ascending = _group_prefs.get_flag (_group, "header-pane-sort-ascending", false);
   int sort_column = _group_prefs.get_int (_group, "header-pane-sort-column", COL_DATE);
@@ -649,7 +645,7 @@ HeaderPane :: set_group (const Quark& new_group)
       save_sort_order (get_group(), _group_prefs, _tree_store);
       _mid_to_row.clear ();
       _tree_store = nullptr;
-      gtk_tree_view_set_model (GTK_TREE_VIEW(_tree_view), NULL);
+      gtk_tree_view_set_model (GTK_TREE_VIEW(_tree_view), nullptr);
     }
 
     _group = new_group;
@@ -657,7 +653,7 @@ HeaderPane :: set_group (const Quark& new_group)
     delete _atree;
     _atree = nullptr;
 
-    char * pch = g_build_filename (g_get_home_dir(), "News", NULL);
+    char * pch = g_build_filename (g_get_home_dir(), "News", nullptr);
     Quark path(_group_prefs.get_string (_group, "default-group-save-path", pch));
     g_free(pch);
 
@@ -753,7 +749,7 @@ HeaderPane :: collapse_selected()
     gtk_tree_view_collapse_row (view, path);
 //    gtk_tree_view_expand_to_path (view, path);
   }
-  g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+  g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
   g_list_free (list);
 }
 }
@@ -770,8 +766,8 @@ HeaderPane :: select_message_id (const Quark& mid, bool do_scroll)
   GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
   gtk_tree_selection_select_path (sel, path);
   if (do_scroll) {
-    gtk_tree_view_set_cursor (view, path, NULL, false);
-    gtk_tree_view_scroll_to_cell (view, path, NULL, true, 0.5f, 0.0f);
+    gtk_tree_view_set_cursor (view, path, nullptr, false);
+    gtk_tree_view_scroll_to_cell (view, path, nullptr, true, 0.5f, 0.0f);
   }
   gtk_tree_path_free (path);
 }
@@ -828,7 +824,7 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     }
 
     g_object_ref(G_OBJECT (_tree_store));
-    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), NULL);
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), nullptr);
     _tree_store->insert_sorted (tmp);
     gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), GTK_TREE_MODEL (_tree_store));
     g_object_unref(G_OBJECT(_tree_store));
@@ -859,7 +855,7 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     g_assert (keep.size() + kill.size() == _mid_to_row.size());
 
     g_object_ref(G_OBJECT (_tree_store));
-    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), NULL);
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), nullptr);
     _tree_store->remove (kill);
     gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), GTK_TREE_MODEL (_tree_store));
     g_object_unref(G_OBJECT(_tree_store));
@@ -1078,7 +1074,7 @@ HeaderPane ::  do_popup_menu (GtkWidget*, GdkEventButton *event, gpointer pane_g
 {
   HeaderPane * self (static_cast<HeaderPane*>(pane_g));
   GtkWidget * menu (self->_action_manager.get_action_widget ("/header-pane-popup"));
-  gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
+  gtk_menu_popup (GTK_MENU(menu), nullptr, nullptr, nullptr, nullptr,
                   (event ? event->button : 0),
                   (event ? event->time : 0));
 }
@@ -1087,7 +1083,7 @@ namespace
 {
   gboolean on_popup_menu (GtkWidget * treeview, gpointer userdata)
   {
-    HeaderPane::do_popup_menu (treeview, NULL, userdata);
+    HeaderPane::do_popup_menu (treeview, nullptr, userdata);
     return true;
   }
 }
@@ -1210,7 +1206,7 @@ HeaderPane :: on_button_pressed (GtkWidget * treeview, GdkEventButton *event, gp
     if (gtk_tree_view_get_path_at_pos (tv,
                                        (gint) event->x,
                                        (gint) event->y,
-                                       &path, NULL, NULL, NULL))
+                                       &path, nullptr, nullptr, nullptr))
     {
       if (!gtk_tree_selection_path_is_selected (selection, path))
       {
@@ -1264,14 +1260,18 @@ namespace
   gboolean on_row_activated_idle (gpointer pane_g)
   {
     HeaderPane * pane (static_cast<HeaderPane*>(pane_g));
-    const Article * a (pane->get_first_selected_article());
+    Article const *a(pane->get_first_selected_article());
     if (a) {
       const size_t lines = a->get_line_count();
-      const bool is_smallish = lines  <= 5000;
-      const bool is_mediumish = lines <= 20000;
-      const bool image_subject = has_image_type_in_subject (*a);
-      const bool is_pictures_newsgroup = pane->get_group().to_view().strstr("pictures")!=nullptr;
-      if (is_smallish || image_subject)
+      bool const is_smallish = lines <= 5000;
+      bool const is_mediumish = lines <= 20000;
+      bool const image_subject = has_image_type_in_subject(*a);
+      bool const is_pictures_newsgroup =
+        pane->get_group().to_view().strstr("pictures") != nullptr;
+      // This is not the number of MIME parts
+      int const part_count = a->get_found_part_count();
+
+      if (is_smallish || image_subject || part_count < 2)
         pane->_action_manager.activate_action ("read-selected-article");
       else if (is_mediumish && is_pictures_newsgroup)
         pane->_action_manager.activate_action ("read-selected-article");
@@ -1569,7 +1569,7 @@ namespace
                                      GdkEventFocus * ,
                                      gpointer        )
   {
-    gtk_widget_override_color (w, GTK_STATE_FLAG_NORMAL, NULL);
+    gtk_widget_override_color (w, GTK_STATE_FLAG_NORMAL, nullptr);
     set_search_entry (w, search_text.c_str());
     return false;
   }
@@ -1664,14 +1664,14 @@ namespace
       set_search_entry (GTK_WIDGET(entry), "");
       refresh_search_entry (GTK_WIDGET(entry));
       search_text.clear ();
-      search_entry_activated (NULL, pane_gpointer);
+      search_entry_activated (nullptr, pane_gpointer);
     }
   }
 
 
   void ellipsize_if_supported (GObject * o)
   {
-    g_object_set (o, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+    g_object_set (o, "ellipsize", PANGO_ELLIPSIZE_END, nullptr);
   }
 }
 
@@ -1683,7 +1683,7 @@ HeaderPane :: build_tree_columns ()
 
   // out with the old columns, if any
   GList * old_columns = gtk_tree_view_get_columns (tree_view);
-  for (GList *l=old_columns; l!=NULL; l=l->next)
+  for (GList *l=old_columns; l!=nullptr; l=l->next)
     gtk_tree_view_remove_column (tree_view, GTK_TREE_VIEW_COLUMN(l->data));
   g_list_free (old_columns);
 
@@ -1701,7 +1701,7 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       col = gtk_tree_view_column_new ();
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 24));
@@ -1716,7 +1716,7 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       col = gtk_tree_view_column_new ();
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 24));
@@ -1730,9 +1730,9 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Subject"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Subject"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 400));
       gtk_tree_view_column_set_resizable (col, true);
@@ -1747,9 +1747,9 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Score"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Score"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 50));
       gtk_tree_view_column_set_resizable (col, true);
@@ -1762,9 +1762,9 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Author"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Author"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 133));
       gtk_tree_view_column_set_resizable (col, true);
@@ -1778,9 +1778,9 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Lines"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Lines"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 60));
       gtk_tree_view_column_set_resizable (col, true);
@@ -1794,9 +1794,9 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Bytes"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Bytes"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 80));
       gtk_tree_view_column_set_resizable (col, true);
@@ -1809,9 +1809,9 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Date"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Date"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 120));
       gtk_tree_view_column_set_resizable (col, true);
@@ -1854,7 +1854,7 @@ HeaderPane :: ~HeaderPane ()
 
   // save the column widths
   GList * columns = gtk_tree_view_get_columns (GTK_TREE_VIEW(_tree_view));
-  for (GList *l=columns; l!=NULL; l=l->next) {
+  for (GList *l=columns; l!=nullptr; l=l->next) {
     GtkTreeViewColumn * col (GTK_TREE_VIEW_COLUMN(l->data));
     const int width = gtk_tree_view_column_get_width (col);
     const char * width_key = (const char*) g_object_get_data (G_OBJECT(col), "column-width-key");
@@ -1876,8 +1876,8 @@ HeaderPane :: create_filter_entry ()
   GtkWidget * entry = gtk_entry_new ();
   _action_manager.disable_accelerators_when_focused (entry);
   g_object_set_data (G_OBJECT(entry), "header-pane", this);
-  g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), NULL);
-  g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), NULL);
+  g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), nullptr);
+  g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), nullptr);
   g_signal_connect (entry, "activate", G_CALLBACK(search_entry_activated), this);
   entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed), this);
 
@@ -1977,16 +1977,18 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   _tree_view (nullptr),
   _tree_store (nullptr),
   _selection_changed_idle_tag (0),
-  _fg(prefs.get_color_str_wo_fallback ("text-color-fg")),
-  _bg(prefs.get_color_str_wo_fallback ("text-color-bg")),
+  _fg(),
+  _bg(),
   _cache (cache),
   _gui (gui),
   _cleared (true)
 {
+  _fg = prefs.get_color_str ("text-color-fg",colors.def_fg.c_str());
+  _bg = prefs.get_color_str ("text-color-bg", colors.def_bg.c_str());
 
   // init the icons
   for (guint i=0; i<ICON_QTY; ++i)
-    _icons[i].pixbuf = gdk_pixbuf_new_from_inline (-1, _icons[i].pixbuf_txt, FALSE, nullptr);
+    _icons[i].pixbuf = load_icon (_icons[i].icon_file);
 
   // initialize the show type...
   const std::string show_type_str (prefs.get_string ("header-pane-show-matching", "articles"));
@@ -2014,8 +2016,8 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   /* intercept ENTER and left/right for article selection */
   g_signal_connect (w, "key-press-event", G_CALLBACK(on_keyboard_button_pressed), this);
 
-  g_signal_connect (w, "row-collapsed", G_CALLBACK(row_collapsed_cb), NULL);
-  g_signal_connect (w, "row-expanded", G_CALLBACK(row_expanded_cb), NULL);
+  g_signal_connect (w, "row-collapsed", G_CALLBACK(row_collapsed_cb), nullptr);
+  g_signal_connect (w, "row-expanded", G_CALLBACK(row_expanded_cb), nullptr);
   g_signal_connect (w, "popup-menu", G_CALLBACK(on_popup_menu), this);
   g_signal_connect (w, "row-activated", G_CALLBACK(on_row_activated), this);
   GtkWidget * scroll = gtk_scrolled_window_new (nullptr, nullptr);
@@ -2094,7 +2096,7 @@ namespace
       select_subpaths (view, model, sel, path);
     }
 
-    g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+    g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
     g_list_free (list);
   }
 }
@@ -2125,7 +2127,7 @@ HeaderPane :: expand_selected ()
     gtk_tree_view_expand_row (view, path, true);
     gtk_tree_view_expand_to_path (view, path);
   }
-  g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+  g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
   g_list_free (list);
 }
 
@@ -2248,8 +2250,8 @@ void operator() (GtkTreeModel* model, GtkTreeIter* iter, const Article&) overrid
       GtkTreePath * path = gtk_tree_model_get_path (model, iter);
       gtk_tree_view_expand_row (_view, path, true);
       gtk_tree_view_expand_to_path (_view, path);
-      gtk_tree_view_set_cursor (_view, path, NULL, FALSE);
-      gtk_tree_view_scroll_to_cell (_view, path, NULL, true, 0.5f, 0.0f);
+      gtk_tree_view_set_cursor (_view, path, nullptr, FALSE);
+      gtk_tree_view_scroll_to_cell (_view, path, nullptr, true, 0.5f, 0.0f);
       gtk_tree_path_free (path);
     }
   };
@@ -2308,7 +2310,7 @@ namespace
     GtkTreeModel * model (nullptr);
     GList * list (gtk_tree_selection_get_selected_rows (sel, &model));
     const bool found (list && gtk_tree_model_get_iter (model, setme, (GtkTreePath*)(list->data)));
-    g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+    g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
     g_list_free (list);
     return found;
   }
@@ -2474,12 +2476,12 @@ HeaderPane :: on_prefs_string_changed (const StringView& key, const StringView&)
 }
 
 void
-HeaderPane :: on_prefs_color_changed  (const StringView& key, const GdkColor&)
+HeaderPane :: on_prefs_color_changed  (const StringView& key, const GdkRGBA&)
 {
   if (key == "text-color-fg" || key == "text-color-bg")
   {
-    _fg = _prefs.get_color_str ("text-color-fg", def_color_fg_str).c_str();
-    _bg = _prefs.get_color_str ("text-color-bg", def_color_str).c_str();
+    _fg = _prefs.get_color_str ("text-color-fg", colors.def_fg).c_str();
+    _bg = _prefs.get_color_str ("text-color-bg", colors.def_bg).c_str();
     refresh_font();
     build_tree_columns();
   }
@@ -2632,7 +2634,7 @@ struct HeaderPane::SimilarWalk: public PanTreeStore::WalkFunctor
 
         retval = (double)common_chars / a.len;
       }
-        else // more than one word, so count common words
+      else // more than one word, so count common words
       {
         StringView tok;
         int str1_words(0), common_words(0);
