@@ -19,18 +19,21 @@
 
 #include <config.h>
 #include <glib/gi18n.h>
-#include "gtk-compat.h"
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <pan/general/debug.h>
 #include <pan/general/macros.h>
 #include <pan/general/file-util.h>
-#include <pan/icons/pan-pixbufs.h>
 #include "hig.h"
+#include <list>
 #include "pad.h"
 #include "pan-file-entry.h"
+#include "pan/gui/load-icon.h"
 #include "prefs-ui.h"
 #include "tango-colors.h"
 #include "url.h"
-#include "gtk-compat.h"
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include "e-charset-dialog.h"
 #include "actions-extern.h"
 
@@ -46,7 +49,6 @@ extern "C" {
 
 #include <algorithm>
 
-#include <pan/general/null.h>
 
 using namespace pan;
 
@@ -95,7 +97,12 @@ namespace pan
       static_cast<CallBackData*>(user_data)->dialog->update_hotkey(user_data);
     }
     gtk_widget_destroy(dialog);
-    delete static_cast<CallBackData*>(user_data);
+
+    // user_data pointer must not be destroyed otherwise user cannot
+    // edit twice the same shortcut without getting a core dump. This
+    // may be flagged as a memory leak, but it's limited.
+
+    //delete static_cast<CallBackData*>(user_data);
   }
 }
 
@@ -108,7 +115,7 @@ PrefsDialog :: update_hotkey (gpointer user_data)
   guint lkey;
   GdkModifierType lmods;
   GtkAccelKey acc_key;
-  const gchar* str = gtk_label_get_text(GTK_LABEL(data->label));
+  gchar const *str = gtk_label_get_text(GTK_LABEL(data->label));
   gtk_accelerator_parse(str, &lkey, &lmods);
 
   acc_key.accel_key = lkey;
@@ -133,25 +140,23 @@ PrefsDialog :: edit_shortkey (gpointer user_data)
   dialog = gtk_dialog_new_with_buttons(_("Grab Key"), GTK_WINDOW(root()),
       GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
+      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, nullptr);
 
   str = g_strdup_printf(
       _("Press the combination of the keys\nyou want to use for \"%s\"."), data->value.c_str());
   label = gtk_label_new(str);
-  gtk_misc_set_padding(GTK_MISC(label), 5, 10);
 
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 		      label,
 		      FALSE, FALSE,
-		      2);
+		      5);
 
   data->label = gtk_label_new("");
-  gtk_misc_set_padding(GTK_MISC(data->label), 5, 10);
 
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 		      data->label,
 		      FALSE, FALSE,
-		      2);
+		      5);
 
   g_signal_connect(dialog, "key-press-event",
             G_CALLBACK(hotkey_key_press_cb), user_data);
@@ -175,12 +180,10 @@ PrefsDialog :: populate_popup (GtkEntry *e, GtkMenu *m)
   gtk_widget_show (mi);
   gtk_menu_shell_prepend (GTK_MENU_SHELL(m), mi);
 
-  GtkWidget * img = gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
-  mi = gtk_image_menu_item_new_with_mnemonic (_("Edit Shortcut"));
+  mi = gtk_menu_item_new_with_mnemonic (_("Edit Shortcut"));
 
   g_signal_connect (mi, "activate", G_CALLBACK(edit_shortkey_cb),
                     g_object_get_data(G_OBJECT(e), "data"));
-  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
   gtk_widget_show_all (mi);
   gtk_menu_shell_prepend (GTK_MENU_SHELL(m), mi);
 }
@@ -189,7 +192,7 @@ namespace pan
 {
 
   std::string get_accel_filename () {
-    char * tmp = g_build_filename (file::get_pan_home().c_str(), "accels.txt", NULL);
+    char * tmp = g_build_filename (file::get_pan_home().c_str(), "accels.txt", nullptr);
     std::string ret (tmp);
     g_free (tmp);
     return ret;
@@ -206,7 +209,7 @@ namespace pan
   {
 
     char* name = static_cast<char*>(gpointer);
-    const char * value = gtk_entry_get_text(e);
+    char const *value = gtk_entry_get_text(e);
 
     gtk_entry_set_icon_activatable(e, GTK_ENTRY_ICON_PRIMARY, false);
     // gtk_entry_set_icon_from_stock(e, GTK_ENTRY_ICON_PRIMARY, nullptr);
@@ -215,7 +218,7 @@ namespace pan
     // empty text
     if (!value || !*value)
     {
-      gtk_entry_set_icon_from_stock(e, GTK_ENTRY_ICON_PRIMARY, nullptr);
+      gtk_entry_set_icon_from_icon_name(e, GTK_ENTRY_ICON_PRIMARY, nullptr);
       gtk_entry_set_icon_tooltip_text (e, GTK_ENTRY_ICON_PRIMARY, nullptr);
       // reset in map and remove accelerator
       GtkAccelKey tmp;
@@ -236,11 +239,11 @@ namespace pan
     acc_key.accel_mods = mod;
 
     if (!gtk_accelerator_valid(acc_key.accel_key, acc_key.accel_mods))
-    {
-      gtk_entry_set_icon_from_stock(e, GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_DIALOG_WARNING);
-      gtk_entry_set_icon_tooltip_text (e, GTK_ENTRY_ICON_PRIMARY, _("Error: Shortcut key is invalid!"));
-      return;
-    }
+      {
+        gtk_entry_set_icon_from_icon_name(e, GTK_ENTRY_ICON_PRIMARY, "dialog-warning");
+        gtk_entry_set_icon_tooltip_text (e, GTK_ENTRY_ICON_PRIMARY, _("Error: Shortcut key is invalid!"));
+        return;
+      }
 
     bool found(false);
 
@@ -253,21 +256,22 @@ namespace pan
 
     if (found)
     {
-      gtk_entry_set_icon_from_stock(e, GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_DIALOG_WARNING);
-      gtk_entry_set_icon_tooltip_text (e, GTK_ENTRY_ICON_PRIMARY, _("Error: Shortcut key already exists!"));
+      gtk_entry_set_icon_from_icon_name(e, GTK_ENTRY_ICON_PRIMARY, "dialog-warning");
+      gtk_entry_set_icon_tooltip_text(e, GTK_ENTRY_ICON_PRIMARY, _("Error: Shortcut key already exists!"));
     }
     else
     {
+      gtk_entry_set_icon_from_icon_name(e, GTK_ENTRY_ICON_PRIMARY, nullptr);
+      gtk_entry_set_icon_tooltip_text(e, GTK_ENTRY_ICON_PRIMARY, nullptr);
       hotkey_data.keys[name] = acc_key;
     }
-
   }
 
-  void process_accels  (gpointer _data,
-                       const gchar *accel_path,
-                       guint accel_key,
-                       GdkModifierType accel_mods,
-                       gboolean changed)
+  void process_accels(gpointer _data,
+                      gchar const *accel_path,
+                      guint accel_key,
+                      GdkModifierType accel_mods,
+                      gboolean changed)
   {
     HotkeyData* data = static_cast<HotkeyData*>(_data);
     GtkAccelKey key;
@@ -319,28 +323,34 @@ namespace pan
 
   void toggled_cb (GtkToggleButton * toggle, gpointer prefs_gpointer)
   {
-    const char * key = (const char*) g_object_get_data (G_OBJECT(toggle), PREFS_KEY);
+    char const *key =
+      (char const *)g_object_get_data(G_OBJECT(toggle), PREFS_KEY);
     if (key)
       static_cast<Prefs*>(prefs_gpointer)->set_flag (key, gtk_toggle_button_get_active(toggle));
   }
 
   void entry_changed_cb (GtkEntry * e, gpointer prefs_gpointer)
   {
-    const char * key = (const char*) g_object_get_data (G_OBJECT(e), PREFS_KEY);
-    const char * val = gtk_entry_get_text (GTK_ENTRY(e));
+    char const *key = (char const *)g_object_get_data(G_OBJECT(e), PREFS_KEY);
+    char const *val = gtk_entry_get_text(GTK_ENTRY(e));
     if (key && val)
       static_cast<Prefs*>(prefs_gpointer)->set_string (key, val);
   }
 
   void set_string_from_radio_cb (GtkToggleButton * toggle, gpointer prefs_gpointer)
   {
-    const char * key = (const char*) g_object_get_data (G_OBJECT(toggle), PREFS_KEY);
-    const char * val = (const char*) g_object_get_data (G_OBJECT(toggle), PREFS_VAL);
+    char const *key =
+      (char const *)g_object_get_data(G_OBJECT(toggle), PREFS_KEY);
+    char const *val =
+      (char const *)g_object_get_data(G_OBJECT(toggle), PREFS_VAL);
     if (key && val && gtk_toggle_button_get_active(toggle))
       static_cast<Prefs*>(prefs_gpointer)->set_string (key, val);
   }
 
-  GtkWidget* new_check_button (const char* mnemonic, const char* key, bool fallback, Prefs& prefs)
+  GtkWidget *new_check_button(char const *mnemonic,
+                              char const *key,
+                              bool fallback,
+                              Prefs &prefs)
   {
     GtkWidget * t = gtk_check_button_new_with_mnemonic (mnemonic);
     g_object_set_data_full (G_OBJECT(t), PREFS_KEY, g_strdup(key), g_free);
@@ -349,7 +359,7 @@ namespace pan
     return t;
   }
 
-  GtkWidget* new_entry (const char* key, const char* fallback, Prefs& prefs)
+  GtkWidget *new_entry(char const *key, char const *fallback, Prefs &prefs)
   {
     GtkWidget * t = gtk_entry_new();
     g_object_set_data_full (G_OBJECT(t), PREFS_KEY, g_strdup(key), g_free);
@@ -358,7 +368,7 @@ namespace pan
     return t;
   }
 
-  GtkWidget* new_hotkey_entry (const char* value, const char* name, gpointer ptr)
+  GtkWidget *new_hotkey_entry(char const *value, char const *name, gpointer ptr)
   {
 
     GtkWidget * t = gtk_entry_new();
@@ -370,14 +380,19 @@ namespace pan
     return t;
   }
 
-  GtkWidget* new_layout_radio (GtkWidget* prev, const guint8* line, const char* value, std::string& cur, Prefs& prefs)
+  GtkWidget *new_layout_radio(GtkWidget *prev,
+                              char const *icon_file,
+                              char const *value,
+                              std::string &cur,
+                              Prefs &prefs)
   {
     GtkWidget * r = prev==nullptr
-      ? gtk_radio_button_new (NULL)
+      ? gtk_radio_button_new (nullptr)
       : gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON(prev));
-    GdkPixbuf * pixbuf = gdk_pixbuf_new_from_inline (-1, line, false, nullptr);
+    GdkPixbuf * pixbuf = load_icon (icon_file);
     GtkWidget * image = gtk_image_new_from_pixbuf (pixbuf);
-    g_object_unref (pixbuf);
+    if (pixbuf != nullptr)
+      g_object_unref (pixbuf);
     gtk_container_add (GTK_CONTAINER(r), image);
     g_object_set_data_full (G_OBJECT(r), PREFS_KEY, g_strdup("pane-layout"), g_free);
     g_object_set_data_full (G_OBJECT(r), PREFS_VAL, g_strdup(value), g_free);
@@ -388,12 +403,13 @@ namespace pan
 
   void spin_value_changed_cb( GtkSpinButton *spin, gpointer data)
   {
-    const char * key = (const char*) g_object_get_data (G_OBJECT(spin), PREFS_KEY);
+    char const *key =
+      (char const *)g_object_get_data(G_OBJECT(spin), PREFS_KEY);
     Prefs *prefs = static_cast<Prefs*>(data);
     prefs->set_int(key, gtk_spin_button_get_value_as_int(spin));
   }
 
-  GtkWidget* new_spin_button (const char *key, int low, int high, Prefs &prefs)
+  GtkWidget *new_spin_button(char const *key, int low, int high, Prefs &prefs)
   {
     guint tm = prefs.get_int(key, low );
     GtkAdjustment *adj = (GtkAdjustment*) gtk_adjustment_new(tm, low, high, 1.0, 1.0, 0.0);
@@ -403,7 +419,11 @@ namespace pan
     return w;
   }
 
-  GtkWidget* new_orient_radio (GtkWidget* prev, const char* label, const char* value, std::string& cur, Prefs& prefs)
+  GtkWidget *new_orient_radio(GtkWidget *prev,
+                              char const *label,
+                              char const *value,
+                              std::string &cur,
+                              Prefs &prefs)
   {
     GtkWidget * r = prev==nullptr
       ? gtk_radio_button_new_with_label (nullptr, label)
@@ -415,18 +435,22 @@ namespace pan
     return r;
   }
 
-  GtkWidget* new_label_with_icon(const char* mnemonic, const char* label, const guint8* line, Prefs& prefs)
+  GtkWidget *new_label_with_icon(char const *mnemonic,
+                                 char const *label,
+                                 char const *icon_file,
+                                 Prefs &prefs)
   {
     std::string what = prefs.get_string("elements-show-tabs", "text");
-    const bool text  = "text"  == what;
-    const bool icons = "icons" == what;
-    const bool both  = "both"  == what;
+    bool const text = "text" == what;
+    bool const icons = "icons" == what;
+    bool const both = "both" == what;
 
     GtkWidget* hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
-    GdkPixbuf * pixbuf = gdk_pixbuf_new_from_inline (-1, line, false, nullptr);
+    GdkPixbuf * pixbuf = load_icon (icon_file);
     GtkWidget * image = gtk_image_new_from_pixbuf (pixbuf);
-    g_object_unref (pixbuf);
-    if (line && (icons || both)) gtk_box_pack_start (GTK_BOX(hbox), image, true, true, 0);
+    if (pixbuf != nullptr)
+      g_object_unref(pixbuf);
+    if (icon_file && (icons || both)) gtk_box_pack_start (GTK_BOX(hbox), image, true, true, 0);
     if (text || both) gtk_box_pack_start (GTK_BOX(hbox), gtk_label_new_with_mnemonic(mnemonic), true, true, 0);
     gtk_widget_set_tooltip_text (hbox, label);
     gtk_widget_show_all(hbox);
@@ -440,13 +464,14 @@ namespace pan
     return res;
   }
 
-  void fill_pref_hotkeys(GtkWidget* t, int& row, Prefs& prefs, gpointer dialog_ptr)
+  std::list<CallBackData*> fill_pref_hotkeys(GtkWidget* t, int& row, Prefs& prefs, gpointer dialog_ptr)
   {
 
     HIG::workarea_add_section_spacer (t, row, hotkey_data.keys.size());
 
     GtkWidget* w, *l;
     gchar* keyval;
+    std::list<CallBackData*> ptr_list;
 
     foreach (keymap_t, hotkey_data.keys, it)
     {
@@ -457,6 +482,9 @@ namespace pan
       stripped = f != std::string::npos ? stripped.substr(f+1,stripped.size()) : stripped;
 
       CallBackData* data = new CallBackData();
+      // store the pointer so it can be destroyed when preferences widget is destroyed
+      ptr_list.push_back(data);
+
       data->dialog = (PrefsDialog*)dialog_ptr;
       if (!it->first.empty()) data->name = it->first;
       data->value = stripped;
@@ -466,12 +494,15 @@ namespace pan
       l = gtk_label_new(label.c_str());
       HIG :: workarea_add_row (t, &row, w, l);
     }
+
+    return ptr_list;
   }
 
   void set_prefs_string_from_editable (GtkEditable * editable, gpointer prefs_gpointer)
   {
     Prefs * prefs (static_cast<Prefs*>(prefs_gpointer));
-    const char * key = (const char*) g_object_get_data (G_OBJECT(editable), PREFS_KEY);
+    char const *key =
+      (char const *)g_object_get_data(G_OBJECT(editable), PREFS_KEY);
     char * val = gtk_editable_get_chars (editable, 0, -1);
     prefs->set_string (key, val);
     g_free (val);
@@ -483,8 +514,8 @@ namespace pan
     GtkWidget * c_parent (gtk_widget_get_parent (GTK_WIDGET(c)));
     GtkWidget * w_parent (gtk_widget_get_parent (GTK_WIDGET(w)));
     GtkTreeModel * model = gtk_combo_box_get_model (c);
-    const int n_rows (gtk_tree_model_iter_n_children (model, NULL));
-    const bool do_show (gtk_combo_box_get_active(c) == (n_rows-1));
+    int const n_rows(gtk_tree_model_iter_n_children(model, nullptr));
+    bool const do_show(gtk_combo_box_get_active(c) == (n_rows - 1));
 
     if (do_show && !w_parent && c_parent) // add it
     {
@@ -505,7 +536,7 @@ namespace pan
 
   void set_prefs_string_from_combo_box_entry (GtkComboBoxText * c, gpointer user_data)
   {
-    const char * key = (const char*) g_object_get_data (G_OBJECT(c), PREFS_KEY);
+    char const *key = (char const *)g_object_get_data(G_OBJECT(c), PREFS_KEY);
     char * val = gtk_combo_box_text_get_active_text (c);
     static_cast<Prefs*>(user_data)->set_string (key, val);
     g_free (val);
@@ -513,9 +544,9 @@ namespace pan
 
   GtkWidget* html_previewer_new (Prefs& prefs)
   {
-    const char * key = "html-previewer";
-//    const std::string editor = prefs.get_string (key, "mutt");
-//    editors.insert (editor);
+    char const *key = "html-previewer";
+    //    const std::string editor = prefs.get_string (key, "mutt");
+    //    editors.insert (editor);
     GtkWidget * c = gtk_combo_box_text_new_with_entry ();
     g_object_set_data_full (G_OBJECT(c), PREFS_KEY, g_strdup(key), g_free);
 //    foreach_const (std::set<std::string>, editors, it)
@@ -530,7 +561,7 @@ namespace pan
   {
     std::set<std::string> editors;
     URL :: get_default_editors (editors);
-    const char * key = "editor";
+    char const *key = "editor";
     const std::string editor = prefs.get_string (key, *editors.begin());
     editors.insert (editor);
     GtkWidget * c = gtk_combo_box_text_new_with_entry ();
@@ -546,12 +577,13 @@ namespace pan
   void set_prefs_string_from_combobox (GtkComboBox * c, gpointer user_data)
   {
     Prefs * prefs (static_cast<Prefs*>(user_data));
-    const char * key = (const char*) g_object_get_data (G_OBJECT(c), PREFS_KEY);
+    char const *key = (char const *)g_object_get_data(G_OBJECT(c), PREFS_KEY);
 
     prefs->_rules_changed = strcmp(key,"rules-");
 
-    const int column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(c), "column"));
-    const int row (gtk_combo_box_get_active (c));
+    int const column =
+      GPOINTER_TO_INT(g_object_get_data(G_OBJECT(c), "column"));
+    int const row(gtk_combo_box_get_active(c));
     GtkTreeModel * m = gtk_combo_box_get_model (c);
     GtkTreeIter i;
     if (gtk_tree_model_iter_nth_child (m, &i, nullptr, row)) {
@@ -562,12 +594,10 @@ namespace pan
     }
   }
 
-  GtkWidget* new_tabs_combo_box (Prefs& prefs,
-                                 const char * mode_key)
+  GtkWidget *new_tabs_combo_box(Prefs &prefs, char const *mode_key)
   {
 
-    const char* strings[3][2] =
-    {
+    char const *strings[3][2] = {
       {N_("Show only icons"), "icons"},
       {N_("Show only text"), "text"},
       {N_("Show icons and text"), "both"},
@@ -588,7 +618,7 @@ namespace pan
     GtkWidget * c = gtk_combo_box_new_with_model (GTK_TREE_MODEL(store));
     GtkCellRenderer * renderer (gtk_cell_renderer_text_new ());
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (c), renderer, true);
-    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (c), renderer, "text", 0, NULL);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (c), renderer, "text", 0, nullptr);
     gtk_combo_box_set_active (GTK_COMBO_BOX(c), sel_index);
     g_object_set_data_full (G_OBJECT(c), PREFS_KEY, g_strdup(mode_key), g_free);
     g_object_set_data (G_OBJECT(c), "column", GINT_TO_POINTER(1));
@@ -599,21 +629,21 @@ namespace pan
     return c;
   }
 
-  GtkWidget* url_handler_new (Prefs& prefs,
-                              const char * mode_key,
-                              const char * mode_fallback,
-                              const char * custom_key,
-                              const char * custom_fallback,
-                              GtkWidget *& setme_mnemonic_target)
+  GtkWidget *url_handler_new(Prefs &prefs,
+                             char const *mode_key,
+                             char const *mode_fallback,
+                             char const *custom_key,
+                             char const *custom_fallback,
+                             GtkWidget *&setme_mnemonic_target)
   {
     // build the combo box...
     const std::string mode (prefs.get_string (mode_key, mode_fallback));
     GtkListStore * store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-    const char* strings[5][2] = { { N_("Use GNOME Preferences"), "gnome" },
-                                  { N_("Use KDE Preferences"), "kde" },
-                                  { N_("Use OS X Preferences"), "mac" },
-                                  { N_("Use Windows Preferences"), "windows" },
-                                  { N_("Custom Command:"), "custom" } };
+    char const *strings[5][2] = {{N_("Use GNOME Preferences"), "gnome"},
+                                 {N_("Use KDE Preferences"), "kde"},
+                                 {N_("Use OS X Preferences"), "mac"},
+                                 {N_("Use Windows Preferences"), "windows"},
+                                 {N_("Custom Command:"), "custom"}};
     int sel_index (0);
     for (size_t i=0; i<G_N_ELEMENTS(strings); ++i) {
       GtkTreeIter iter;
@@ -626,7 +656,7 @@ namespace pan
     GtkWidget * c = gtk_combo_box_new_with_model (GTK_TREE_MODEL(store));
     GtkCellRenderer * renderer (gtk_cell_renderer_text_new ());
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (c), renderer, true);
-    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (c), renderer, "text", 0, NULL);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (c), renderer, "text", 0, nullptr);
     gtk_combo_box_set_active (GTK_COMBO_BOX(c), sel_index);
     g_object_set_data_full (G_OBJECT(c), PREFS_KEY, g_strdup(mode_key), g_free);
     g_object_set_data (G_OBJECT(c), "column", GINT_TO_POINTER(1));
@@ -657,21 +687,21 @@ namespace pan
     "Scores from 1 to 4999:"
     "Scores from -9998 to -1:"
 */
-  GtkWidget* score_handler_new (Prefs& prefs,
-                              const char * mode_key,
-                              const char * mode_fallback,
-                              GtkWidget *& setme_mnemonic_target)
+  GtkWidget *score_handler_new(Prefs &prefs,
+                               char const *mode_key,
+                               char const *mode_fallback,
+                               GtkWidget *&setme_mnemonic_target)
   {
     // build the combo box...
     const std::string mode (prefs.get_string (mode_key, mode_fallback));
     GtkListStore * store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-    const char* strings[7][2] = { { N_("Disabled"),"never" },
-                                  { N_("Only new (score == 0)"),"new" },
-                                  { N_("9999 or more"), "watched" },
-                                  { N_("5000 to 9998"), "high" },
-                                  { N_("1 to 4999"),    "medium" },
-                                  { N_("-9998 to -1"),  "low" },
-                                  { N_("-9999 or less"),"ignored" }};
+    char const *strings[7][2] = {{N_("Disabled"), "never"},
+                                 {N_("Only new (score == 0)"), "new"},
+                                 {N_("9999 or more"), "watched"},
+                                 {N_("5000 to 9998"), "high"},
+                                 {N_("1 to 4999"), "medium"},
+                                 {N_("-9998 to -1"), "low"},
+                                 {N_("-9999 or less"), "ignored"}};
     int sel_index (0);
     for (size_t i=0; i<G_N_ELEMENTS(strings); ++i) {
       GtkTreeIter iter;
@@ -684,7 +714,7 @@ namespace pan
     GtkWidget * c = gtk_combo_box_new_with_model (GTK_TREE_MODEL(store));
     GtkCellRenderer * renderer (gtk_cell_renderer_text_new ());
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (c), renderer, true);
-    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (c), renderer, "text", 0, NULL);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (c), renderer, "text", 0, nullptr);
     gtk_combo_box_set_active (GTK_COMBO_BOX(c), sel_index);
     g_object_set_data_full (G_OBJECT(c), PREFS_KEY, g_strdup(mode_key), g_free);
     g_object_set_data (G_OBJECT(c), "column", GINT_TO_POINTER(1));
@@ -696,17 +726,18 @@ namespace pan
 
   void font_set_cb (GtkFontButton* b, gpointer prefs_gpointer)
   {
-    const char * key = (const char*) g_object_get_data (G_OBJECT(b), PREFS_KEY);
-    const char * val = gtk_font_button_get_font_name  (b);
+    char const *key = (char const *)g_object_get_data(G_OBJECT(b), PREFS_KEY);
+    char const *val = gtk_font_chooser_get_font((GtkFontChooser*)b);
     if (key && *key && val && *val)
       static_cast<Prefs*>(prefs_gpointer)->set_string (key, val);
   }
 
-  GtkWidget* new_font_button (const char* key, const char * fallback, Prefs& prefs)
+  GtkWidget *new_font_button(char const *key,
+                             char const *fallback,
+                             Prefs &prefs)
   {
     const std::string val (prefs.get_string (key, fallback));
-    GtkWidget * b = gtk_font_button_new ();
-    gtk_font_button_set_font_name (GTK_FONT_BUTTON(b), val.c_str());
+    GtkWidget * b = gtk_font_button_new_with_font (val.c_str());
     g_object_set_data_full (G_OBJECT(b), PREFS_KEY, g_strdup(key), g_free);
     g_signal_connect (b, "font-set", G_CALLBACK(font_set_cb), &prefs);
     return b;
@@ -714,25 +745,26 @@ namespace pan
 
   void color_set_cb (GtkColorButton* b, gpointer prefs_gpointer)
   {
-    const char * key = (const char*) g_object_get_data (G_OBJECT(b), PREFS_KEY);
-    GdkColor val;
-    gtk_color_button_get_color (b, &val);
+    char const *key = (char const *)g_object_get_data(G_OBJECT(b), PREFS_KEY);
+    GdkRGBA val;
+    gtk_color_chooser_get_rgba((GtkColorChooser*)b, &val);
     if (key && *key)
       static_cast<Prefs*>(prefs_gpointer)->set_color (key, val);
   }
 
-  GtkWidget* new_color_button (const char* key, const char * fallback, Prefs& prefs)
+  GtkWidget *new_color_button(char const *key,
+                              char const *fallback,
+                              Prefs &prefs)
   {
-    const GdkColor val (prefs.get_color (key, fallback));
-    GtkWidget * b = gtk_color_button_new_with_color (&val);
+    GdkRGBA rgba_val (prefs.get_color (key, fallback));;
+    GtkWidget * b = gtk_color_button_new_with_rgba (&rgba_val);
     g_object_set_data_full (G_OBJECT(b), PREFS_KEY, g_strdup(key), g_free);
     g_signal_connect (b, "color-set", G_CALLBACK(color_set_cb), &prefs);
     return b;
   }
 }
 
-void
-PrefsDialog :: update_default_charset_label(const StringView& value)
+void PrefsDialog ::update_default_charset_label(StringView const &value)
 {
   char buf[256];
   g_snprintf(buf, sizeof(buf),_("Select default <u>global</u> character set. Current setting: <b>%s</b>."),
@@ -741,8 +773,8 @@ PrefsDialog :: update_default_charset_label(const StringView& value)
   gtk_widget_show_all(charset_label);
 }
 
-void
-PrefsDialog :: on_prefs_string_changed (const StringView& key, const StringView& value)
+void PrefsDialog ::on_prefs_string_changed(StringView const &key,
+                                           StringView const &value)
 {
 
   if (key == "default-charset")
@@ -752,8 +784,7 @@ PrefsDialog :: on_prefs_string_changed (const StringView& key, const StringView&
   }
 }
 
-void
-PrefsDialog :: on_prefs_flag_changed (const StringView& key, bool value)
+void PrefsDialog ::on_prefs_flag_changed(StringView const &key, bool value)
 {
 
   if (key == "allow-multiple-instances")
@@ -894,19 +925,19 @@ namespace
     info->sel = sel;
     g_object_weak_ref (G_OBJECT(view), (GWeakNotify)g_free, info);
 
-    GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TOGGLE, NULL));
-    GtkTreeViewColumn * col = gtk_tree_view_column_new_with_attributes (_("Enabled"), r, "active", 0, NULL);
+    GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TOGGLE, nullptr));
+    GtkTreeViewColumn * col = gtk_tree_view_column_new_with_attributes (_("Enabled"), r, "active", 0, nullptr);
     gtk_tree_view_append_column (GTK_TREE_VIEW(view), col);
     g_signal_connect (r, "toggled", G_CALLBACK(header_col_enabled_toggled_cb), info);
-    r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, NULL));
-    col = gtk_tree_view_column_new_with_attributes (_("Column Name"), r, "text", 2, NULL);
+    r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, nullptr));
+    col = gtk_tree_view_column_new_with_attributes (_("Column Name"), r, "text", 2, nullptr);
     gtk_tree_view_append_column (GTK_TREE_VIEW(view), col);
     gtk_tree_selection_set_mode (sel, GTK_SELECTION_BROWSE);
     GtkTreeIter iter;
     gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
     gtk_tree_selection_select_iter (sel, &iter);
 
-    GtkWidget * f = gtk_frame_new (NULL);
+    GtkWidget * f = gtk_frame_new (nullptr);
     gtk_frame_set_shadow_type (GTK_FRAME(f), GTK_SHADOW_IN);
     gtk_container_add (GTK_CONTAINER(f), view);
     GtkWidget * hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, PAD);
@@ -925,11 +956,21 @@ namespace
 
   void font_toggled_cb (GtkToggleButton * tb, gpointer user_data)
   {
-    const bool active (gtk_toggle_button_get_active (tb));
+    bool const active(gtk_toggle_button_get_active(tb));
     gtk_widget_set_sensitive (GTK_WIDGET(user_data), active);
   }
 }
 
+PrefsDialog :: ~PrefsDialog ()
+{
+  // free pointer created when constructing shortcut editor widget
+  while(!shortcut_ptr_list.empty()) {
+    free(shortcut_ptr_list.back());
+    shortcut_ptr_list.pop_back();
+  }
+}
+
+
 PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
   _prefs (prefs)
 {
@@ -940,7 +981,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
   GtkWidget * dialog = gtk_dialog_new_with_buttons (_("Pan: Preferences"), parent,
                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                                     GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-                                                    NULL);
+                                                    nullptr);
   gtk_window_set_role (GTK_WINDOW(dialog), "pan-preferences-dialog");
   g_signal_connect (dialog, "response", G_CALLBACK(response_cb), this);
   g_signal_connect_swapped (dialog, "destroy", G_CALLBACK(delete_prefs_dialog), this);
@@ -994,12 +1035,10 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_wide_control (t, &row, w);
     w = new_spin_button ("cache-size-megs", 10, 1024*16, prefs);
     l = gtk_label_new(_("Size of article cache (in MiB):"));
-    gtk_misc_set_alignment (GTK_MISC(l), 0.0, 0.5);
     gtk_label_set_mnemonic_widget(GTK_LABEL(l), w);
     HIG::workarea_add_row (t, &row, l, w);
     w = new_entry ("cache-file-extension", "msg", prefs);
     l = gtk_label_new(_("File extension for cached articles: "));
-    gtk_misc_set_alignment (GTK_MISC(l), 0.0, 0.5);
     HIG :: workarea_add_row (t, &row, l, w);
 
     HIG::workarea_add_section_divider (t, &row);
@@ -1008,7 +1047,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_wide_control (t, &row, w);
 
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Behavior"), _("Behavior"), icon_prefs_behavior, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Behavior"), _("Behavior"), "icon_prefs_behavior.png", prefs));
 
   // pane options
   row = 0;
@@ -1021,16 +1060,15 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_wide_control (t, &row, w);
     HIG::workarea_add_section_divider (t, &row);
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Panes"), _("Panes"), icon_prefs_panes, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Panes"), _("Panes"), "icon_prefs_panes.png", prefs));
 
   //charset
   row = 0;
   t = HIG :: workarea_create ();
     HIG :: workarea_add_section_spacer (t, row, 1);
     HIG :: workarea_add_section_title (t, &row, _("Language Settings"));
-    w = gtk_button_new_from_stock (GTK_STOCK_SELECT_FONT);
-    l = charset_label = gtk_label_new (NULL);
-    gtk_misc_set_alignment (GTK_MISC(l), 0.0, 0.0);
+    w = gtk_button_new_with_label (_("Font"));
+    l = charset_label = gtk_label_new (nullptr);
     update_default_charset_label(_prefs.get_string("default-charset","UTF-8"));
     g_signal_connect (w, "clicked", G_CALLBACK(select_prefs_charset_cb), this);
     HIG::workarea_add_row (t, &row, w, l);
@@ -1038,8 +1076,6 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     // systray and notify popup
     HIG :: workarea_add_section_title (t, &row, _("System Tray Behavior"));
     HIG :: workarea_add_section_spacer (t, row, 3);
-    w = new_check_button (_("Minimize to tray"), "status-icon", false, prefs);
-    HIG :: workarea_add_wide_control (t, &row, w);
     w = new_check_button (_("Start Pan minimized"), "start-minimized", false, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
 #ifdef HAVE_LIBNOTIFY
@@ -1080,7 +1116,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_wide_control (t, &row, w);
 
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Miscellaneous"), _("Miscellaneous"), icon_prefs_extras, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Miscellaneous"), _("Miscellaneous"), "icon_prefs_extras.png", prefs));
 
   // Layout
   row = 0;
@@ -1089,15 +1125,15 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     std::string cur = _prefs.get_string ("pane-layout", "stacked-right");
     HIG :: workarea_add_section_spacer (t, row, 1);
     h = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, PAD);
-    w = new_layout_radio (nullptr, icon_layout_1, "stacked-top", cur, prefs);
+    w = new_layout_radio (nullptr, "icon_layout_1.png", "stacked-top", cur, prefs);
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
-    w = new_layout_radio (w, icon_layout_2, "stacked-bottom", cur, prefs);
+    w = new_layout_radio (w, "icon_layout_2.png", "stacked-bottom", cur, prefs);
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
-    w = new_layout_radio (w, icon_layout_3, "stacked-left", cur, prefs);
+    w = new_layout_radio (w, "icon_layout_3.png", "stacked-left", cur, prefs);
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
-    w = new_layout_radio (w, icon_layout_4, "stacked-right", cur, prefs);
+    w = new_layout_radio (w, "icon_layout_4.png", "stacked-right", cur, prefs);
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
-    w = new_layout_radio (w, icon_layout_5, "stacked-vertical", cur, prefs);
+    w = new_layout_radio (w, "icon_layout_5.png", "stacked-vertical", cur, prefs);
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
     HIG::workarea_add_wide_control (t, &row, h);
   HIG :: workarea_add_section_divider (t, &row);
@@ -1118,7 +1154,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_wide_control (t, &row, w);
 
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Layout"), _("Layout"), icon_prefs_layout, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Layout"), _("Layout"), "icon_prefs_layout.png", prefs));
 
   // Headers
   row = 0;
@@ -1127,7 +1163,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_section_spacer(t, row, 1);
     HIG :: workarea_add_wide_control (t, &row, header_columns_layout_new (prefs));
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Headers"), _("Headers"), icon_prefs_headers, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Headers"), _("Headers"), "icon_prefs_headers.png", prefs));
 
   // customizable actions
   row = 0;
@@ -1146,13 +1182,13 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     w = score_handler_new (prefs, "rules-delete-value", "never", b);
     h = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, PAD);
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
-    gtk_box_pack_start (GTK_BOX(h), gtk_label_new(NULL), false, false, 0);
+    gtk_box_pack_start (GTK_BOX(h), gtk_label_new(nullptr), false, false, 0);
     HIG :: workarea_add_row (t, &row, _("_Delete articles scoring at: "), h);
 
     w = score_handler_new (prefs, "rules-mark-read-value", "never", b);
     h = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, PAD);
     gtk_box_pack_start (GTK_BOX(h), w, false, false, 0);
-    gtk_box_pack_start (GTK_BOX(h), gtk_label_new(NULL), false, false, 0);
+    gtk_box_pack_start (GTK_BOX(h), gtk_label_new(nullptr), false, false, 0);
     HIG :: workarea_add_row (t, &row, _("Mark articles read scoring at: "), h);
 
     w = score_handler_new (prefs, "rules-autocache-value", "never", b);
@@ -1168,7 +1204,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_row (t, &row, _("Download attachments of articles scoring at: "), h);
 
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Actions"), _("Actions"), icon_prefs_actions, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Actions"), _("Actions"), "icon_prefs_actions.png", prefs));
 
   // Fonts
   row = 0;
@@ -1194,12 +1230,12 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     b = new_font_button ("monospace-font", "Monospace 10", prefs);
     HIG :: workarea_add_row (t, &row, l, b);
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Fonts"), _("Fonts"), icon_prefs_fonts, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Fonts"), _("Fonts"), "icon_prefs_fonts.png", prefs));
 
   // default color theme's Colors
-  const PanColors& colors (PanColors::get());
-  const char* def_color_str (colors.def_bg.c_str());
-  const char* def_color_fg_str (colors.def_fg.c_str());
+  PanColors const &colors(PanColors::get());
+  char const *def_color_str(colors.def_bg.c_str());
+  char const *def_color_fg_str(colors.def_fg.c_str());
 
   row = 0;
   t = HIG :: workarea_create ();
@@ -1296,7 +1332,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_row (t, &row, _("Text Color:"), h);
   HIG :: workarea_finish (t, &row);
 
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Colors"), _("Colors"), icon_prefs_colors, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Colors"), _("Colors"), "icon_prefs_colors.png", prefs));
 
   // Applications
   row = 0;
@@ -1306,6 +1342,9 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     w = url_handler_new (prefs, "browser-mode", URL::get_environment(),
                                 "custom-browser", "firefox", b);
     HIG :: workarea_add_row (t, &row, _("_Web browser:"), w);
+    w = url_handler_new (prefs, "gemini-mode", URL::get_environment(),
+                                "custom-gemini", "lagrange", b);
+    HIG :: workarea_add_row (t, &row, _("_Gemini client:"), w);
     w = url_handler_new (prefs, "mailer-mode", URL::get_environment(),
                                 "custom-mailer", "thunderbird", b);
     HIG :: workarea_add_row (t, &row, _("_Mail reader:"), w);
@@ -1314,7 +1353,7 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     w = html_previewer_new (prefs);
     HIG :: workarea_add_row (t, &row, _("_HTML previewer:"), w);
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Applications"), _("Applications"), icon_prefs_applications, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Applications"), _("Applications"), "icon_prefs_applications.png", prefs));
 
   // Upload Options
   row = 0;
@@ -1328,18 +1367,18 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
   HIG::workarea_add_row (t, &row, l, w);
 
   HIG :: workarea_finish (t, &row);
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Upload"), _("Upload"), icon_prefs_upload, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Upload"), _("Upload"), "icon_prefs_upload.png", prefs));
 
   // Hotkeys
   row = 0;
   t = HIG :: workarea_create ();
-  fill_pref_hotkeys(t, row, _prefs, this);
+  shortcut_ptr_list = fill_pref_hotkeys(t, row, _prefs, this);
 
   HIG :: workarea_finish (t, &row);
 
-  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Shortcuts"), _("Shortcuts"), icon_prefs_hotkeys, prefs));
+  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, new_label_with_icon(_("_Shortcuts"), _("Shortcuts"), "icon_prefs_hotkeys.png", prefs));
 
-  GtkWidget* scroll = gtk_scrolled_window_new (NULL, NULL);
+  GtkWidget* scroll = gtk_scrolled_window_new (nullptr, nullptr);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
                                   GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
